@@ -1,7 +1,7 @@
 import os
 from contextlib import asynccontextmanager
 from pathlib import Path
-from typing import AsyncGenerator, AsyncIterator, Callable, Generator
+from typing import AsyncGenerator, AsyncIterator, Generator
 
 import anyio
 import pytest
@@ -20,8 +20,7 @@ from starlette.staticfiles import StaticFiles
 from starlette.testclient import TestClient
 from starlette.types import ASGIApp, Receive, Scope, Send
 from starlette.websockets import WebSocket
-
-TestClientFactory = Callable[..., TestClient]
+from tests.types import TestClientFactory
 
 
 async def error_500(request: Request, exc: HTTPException) -> JSONResponse:
@@ -463,8 +462,7 @@ def test_decorator_deprecations() -> None:
 
         async def middleware(
             request: Request, call_next: RequestResponseEndpoint
-        ) -> None:
-            ...  # pragma: no cover
+        ) -> None: ...  # pragma: no cover
 
         app.middleware("http")(middleware)
         assert len(record) == 1
@@ -494,8 +492,7 @@ def test_decorator_deprecations() -> None:
         )
     ) as record:
 
-        async def startup() -> None:
-            ...  # pragma: no cover
+        async def startup() -> None: ...  # pragma: no cover
 
         app.on_event("startup")(startup)
         assert len(record) == 1
