@@ -1,7 +1,7 @@
 from __future__ import annotations
 
 import sys
-from typing import Any, Callable, Iterator
+from typing import Any, Iterator
 
 import anyio
 import pytest
@@ -9,10 +9,8 @@ import pytest
 from starlette.datastructures import Address, State
 from starlette.requests import ClientDisconnect, Request
 from starlette.responses import JSONResponse, PlainTextResponse, Response
-from starlette.testclient import TestClient
 from starlette.types import Message, Receive, Scope, Send
-
-TestClientFactory = Callable[..., TestClient]
+from tests.types import TestClientFactory
 
 
 def test_request_url(test_client_factory: TestClientFactory) -> None:
@@ -269,8 +267,8 @@ def test_request_disconnect(
 
 def test_request_is_disconnected(test_client_factory: TestClientFactory) -> None:
     """
-    If a client disconnect occurs while reading request body
-    then ClientDisconnect should be raised.
+    If a client disconnect occurs after reading request body
+    then request will be set disconnected properly.
     """
     disconnected_after_response = None
 
@@ -278,15 +276,15 @@ def test_request_is_disconnected(test_client_factory: TestClientFactory) -> None
         nonlocal disconnected_after_response
 
         request = Request(scope, receive)
-        await request.body()
+        body = await request.body()
         disconnected = await request.is_disconnected()
-        response = JSONResponse({"disconnected": disconnected})
+        response = JSONResponse({"body": body.decode(), "disconnected": disconnected})
         await response(scope, receive, send)
         disconnected_after_response = await request.is_disconnected()
 
     client = test_client_factory(app)
-    response = client.get("/")
-    assert response.json() == {"disconnected": False}
+    response = client.post("/", content="foo")
+    assert response.json() == {"body": "foo", "disconnected": False}
     assert disconnected_after_response
 
 
