@@ -1,17 +1,19 @@
 import sys
-from typing import Any, Callable, MutableMapping
+from typing import Any, MutableMapping
 
 import anyio
 import pytest
 from anyio.abc import ObjectReceiveStream, ObjectSendStream
 
 from starlette import status
-from starlette.testclient import TestClient
+from starlette.responses import Response
+from starlette.testclient import WebSocketDenialResponse
 from starlette.types import Message, Receive, Scope, Send
 from starlette.websockets import WebSocket, WebSocketDisconnect, WebSocketState
+from tests.types import TestClientFactory
 
 
-def test_websocket_url(test_client_factory: Callable[..., TestClient]):
+def test_websocket_url(test_client_factory: TestClientFactory) -> None:
     async def app(scope: Scope, receive: Receive, send: Send) -> None:
         websocket = WebSocket(scope, receive=receive, send=send)
         await websocket.accept()
@@ -24,7 +26,7 @@ def test_websocket_url(test_client_factory: Callable[..., TestClient]):
         assert data == {"url": "ws://testserver/123?a=abc"}
 
 
-def test_websocket_binary_json(test_client_factory: Callable[..., TestClient]):
+def test_websocket_binary_json(test_client_factory: TestClientFactory) -> None:
     async def app(scope: Scope, receive: Receive, send: Send) -> None:
         websocket = WebSocket(scope, receive=receive, send=send)
         await websocket.accept()
@@ -40,8 +42,8 @@ def test_websocket_binary_json(test_client_factory: Callable[..., TestClient]):
 
 
 def test_websocket_ensure_unicode_on_send_json(
-    test_client_factory: Callable[..., TestClient],
-):
+    test_client_factory: TestClientFactory,
+) -> None:
     async def app(scope: Scope, receive: Receive, send: Send) -> None:
         websocket = WebSocket(scope, receive=receive, send=send)
 
@@ -57,7 +59,7 @@ def test_websocket_ensure_unicode_on_send_json(
         assert data == '{"test":"数据"}'
 
 
-def test_websocket_query_params(test_client_factory: Callable[..., TestClient]):
+def test_websocket_query_params(test_client_factory: TestClientFactory) -> None:
     async def app(scope: Scope, receive: Receive, send: Send) -> None:
         websocket = WebSocket(scope, receive=receive, send=send)
         query_params = dict(websocket.query_params)
@@ -75,7 +77,7 @@ def test_websocket_query_params(test_client_factory: Callable[..., TestClient]):
     any(module in sys.modules for module in ("brotli", "brotlicffi")),
     reason='urllib3 includes "br" to the "accept-encoding" headers.',
 )
-def test_websocket_headers(test_client_factory: Callable[..., TestClient]):
+def test_websocket_headers(test_client_factory: TestClientFactory) -> None:
     async def app(scope: Scope, receive: Receive, send: Send) -> None:
         websocket = WebSocket(scope, receive=receive, send=send)
         headers = dict(websocket.headers)
@@ -98,7 +100,7 @@ def test_websocket_headers(test_client_factory: Callable[..., TestClient]):
         assert data == {"headers": expected_headers}
 
 
-def test_websocket_port(test_client_factory: Callable[..., TestClient]):
+def test_websocket_port(test_client_factory: TestClientFactory) -> None:
     async def app(scope: Scope, receive: Receive, send: Send) -> None:
         websocket = WebSocket(scope, receive=receive, send=send)
         await websocket.accept()
@@ -112,8 +114,8 @@ def test_websocket_port(test_client_factory: Callable[..., TestClient]):
 
 
 def test_websocket_send_and_receive_text(
-    test_client_factory: Callable[..., TestClient],
-):
+    test_client_factory: TestClientFactory,
+) -> None:
     async def app(scope: Scope, receive: Receive, send: Send) -> None:
         websocket = WebSocket(scope, receive=receive, send=send)
         await websocket.accept()
@@ -129,8 +131,8 @@ def test_websocket_send_and_receive_text(
 
 
 def test_websocket_send_and_receive_bytes(
-    test_client_factory: Callable[..., TestClient],
-):
+    test_client_factory: TestClientFactory,
+) -> None:
     async def app(scope: Scope, receive: Receive, send: Send) -> None:
         websocket = WebSocket(scope, receive=receive, send=send)
         await websocket.accept()
@@ -146,8 +148,8 @@ def test_websocket_send_and_receive_bytes(
 
 
 def test_websocket_send_and_receive_json(
-    test_client_factory: Callable[..., TestClient],
-):
+    test_client_factory: TestClientFactory,
+) -> None:
     async def app(scope: Scope, receive: Receive, send: Send) -> None:
         websocket = WebSocket(scope, receive=receive, send=send)
         await websocket.accept()
@@ -162,7 +164,7 @@ def test_websocket_send_and_receive_json(
         assert data == {"message": {"hello": "world"}}
 
 
-def test_websocket_iter_text(test_client_factory: Callable[..., TestClient]):
+def test_websocket_iter_text(test_client_factory: TestClientFactory) -> None:
     async def app(scope: Scope, receive: Receive, send: Send) -> None:
         websocket = WebSocket(scope, receive=receive, send=send)
         await websocket.accept()
@@ -176,7 +178,7 @@ def test_websocket_iter_text(test_client_factory: Callable[..., TestClient]):
         assert data == "Message was: Hello, world!"
 
 
-def test_websocket_iter_bytes(test_client_factory: Callable[..., TestClient]):
+def test_websocket_iter_bytes(test_client_factory: TestClientFactory) -> None:
     async def app(scope: Scope, receive: Receive, send: Send) -> None:
         websocket = WebSocket(scope, receive=receive, send=send)
         await websocket.accept()
@@ -190,7 +192,7 @@ def test_websocket_iter_bytes(test_client_factory: Callable[..., TestClient]):
         assert data == b"Message was: Hello, world!"
 
 
-def test_websocket_iter_json(test_client_factory: Callable[..., TestClient]):
+def test_websocket_iter_json(test_client_factory: TestClientFactory) -> None:
     async def app(scope: Scope, receive: Receive, send: Send) -> None:
         websocket = WebSocket(scope, receive=receive, send=send)
         await websocket.accept()
@@ -204,17 +206,17 @@ def test_websocket_iter_json(test_client_factory: Callable[..., TestClient]):
         assert data == {"message": {"hello": "world"}}
 
 
-def test_websocket_concurrency_pattern(test_client_factory: Callable[..., TestClient]):
+def test_websocket_concurrency_pattern(test_client_factory: TestClientFactory) -> None:
     stream_send: ObjectSendStream[MutableMapping[str, Any]]
     stream_receive: ObjectReceiveStream[MutableMapping[str, Any]]
     stream_send, stream_receive = anyio.create_memory_object_stream()
 
-    async def reader(websocket: WebSocket):
+    async def reader(websocket: WebSocket) -> None:
         async with stream_send:
             async for data in websocket.iter_json():
                 await stream_send.send(data)
 
-    async def writer(websocket: WebSocket):
+    async def writer(websocket: WebSocket) -> None:
         async with stream_receive:
             async for message in stream_receive:
                 await websocket.send_json(message)
@@ -234,7 +236,7 @@ def test_websocket_concurrency_pattern(test_client_factory: Callable[..., TestCl
         assert data == {"hello": "world"}
 
 
-def test_client_close(test_client_factory: Callable[..., TestClient]):
+def test_client_close(test_client_factory: TestClientFactory) -> None:
     close_code = None
     close_reason = None
 
@@ -256,7 +258,7 @@ def test_client_close(test_client_factory: Callable[..., TestClient]):
 
 
 @pytest.mark.anyio
-async def test_client_disconnect_on_send():
+async def test_client_disconnect_on_send() -> None:
     async def app(scope: Scope, receive: Receive, send: Send) -> None:
         websocket = WebSocket(scope, receive=receive, send=send)
         await websocket.accept()
@@ -270,14 +272,14 @@ async def test_client_disconnect_on_send():
             return
         # Simulate the exception the server would send to the application when the
         # client disconnects.
-        raise IOError
+        raise OSError
 
     with pytest.raises(WebSocketDisconnect) as ctx:
         await app({"type": "websocket", "path": "/"}, receive, send)
     assert ctx.value.code == status.WS_1006_ABNORMAL_CLOSURE
 
 
-def test_application_close(test_client_factory: Callable[..., TestClient]):
+def test_application_close(test_client_factory: TestClientFactory) -> None:
     async def app(scope: Scope, receive: Receive, send: Send) -> None:
         websocket = WebSocket(scope, receive=receive, send=send)
         await websocket.accept()
@@ -290,9 +292,11 @@ def test_application_close(test_client_factory: Callable[..., TestClient]):
         assert exc.value.code == status.WS_1001_GOING_AWAY
 
 
-def test_rejected_connection(test_client_factory: Callable[..., TestClient]):
+def test_rejected_connection(test_client_factory: TestClientFactory) -> None:
     async def app(scope: Scope, receive: Receive, send: Send) -> None:
         websocket = WebSocket(scope, receive=receive, send=send)
+        msg = await websocket.receive()
+        assert msg == {"type": "websocket.connect"}
         await websocket.close(status.WS_1001_GOING_AWAY)
 
     client = test_client_factory(app)
@@ -302,7 +306,112 @@ def test_rejected_connection(test_client_factory: Callable[..., TestClient]):
     assert exc.value.code == status.WS_1001_GOING_AWAY
 
 
-def test_subprotocol(test_client_factory: Callable[..., TestClient]):
+def test_send_denial_response(test_client_factory: TestClientFactory) -> None:
+    async def app(scope: Scope, receive: Receive, send: Send) -> None:
+        websocket = WebSocket(scope, receive=receive, send=send)
+        msg = await websocket.receive()
+        assert msg == {"type": "websocket.connect"}
+        response = Response(status_code=404, content="foo")
+        await websocket.send_denial_response(response)
+
+    client = test_client_factory(app)
+    with pytest.raises(WebSocketDenialResponse) as exc:
+        with client.websocket_connect("/"):
+            pass  # pragma: no cover
+    assert exc.value.status_code == 404
+    assert exc.value.content == b"foo"
+
+
+def test_send_response_multi(test_client_factory: TestClientFactory) -> None:
+    async def app(scope: Scope, receive: Receive, send: Send) -> None:
+        websocket = WebSocket(scope, receive=receive, send=send)
+        msg = await websocket.receive()
+        assert msg == {"type": "websocket.connect"}
+        await websocket.send(
+            {
+                "type": "websocket.http.response.start",
+                "status": 404,
+                "headers": [(b"content-type", b"text/plain"), (b"foo", b"bar")],
+            }
+        )
+        await websocket.send(
+            {
+                "type": "websocket.http.response.body",
+                "body": b"hard",
+                "more_body": True,
+            }
+        )
+        await websocket.send(
+            {
+                "type": "websocket.http.response.body",
+                "body": b"body",
+            }
+        )
+
+    client = test_client_factory(app)
+    with pytest.raises(WebSocketDenialResponse) as exc:
+        with client.websocket_connect("/"):
+            pass  # pragma: no cover
+    assert exc.value.status_code == 404
+    assert exc.value.content == b"hardbody"
+    assert exc.value.headers["foo"] == "bar"
+
+
+def test_send_response_unsupported(test_client_factory: TestClientFactory) -> None:
+    async def app(scope: Scope, receive: Receive, send: Send) -> None:
+        del scope["extensions"]["websocket.http.response"]
+        websocket = WebSocket(scope, receive=receive, send=send)
+        msg = await websocket.receive()
+        assert msg == {"type": "websocket.connect"}
+        response = Response(status_code=404, content="foo")
+        with pytest.raises(
+            RuntimeError,
+            match="The server doesn't support the Websocket Denial Response extension.",
+        ):
+            await websocket.send_denial_response(response)
+        await websocket.close()
+
+    client = test_client_factory(app)
+    with pytest.raises(WebSocketDisconnect) as exc:
+        with client.websocket_connect("/"):
+            pass  # pragma: no cover
+    assert exc.value.code == status.WS_1000_NORMAL_CLOSURE
+
+
+def test_send_response_duplicate_start(test_client_factory: TestClientFactory) -> None:
+    async def app(scope: Scope, receive: Receive, send: Send) -> None:
+        websocket = WebSocket(scope, receive=receive, send=send)
+        msg = await websocket.receive()
+        assert msg == {"type": "websocket.connect"}
+        response = Response(status_code=404, content="foo")
+        await websocket.send(
+            {
+                "type": "websocket.http.response.start",
+                "status": response.status_code,
+                "headers": response.raw_headers,
+            }
+        )
+        await websocket.send(
+            {
+                "type": "websocket.http.response.start",
+                "status": response.status_code,
+                "headers": response.raw_headers,
+            }
+        )
+
+    client = test_client_factory(app)
+    with pytest.raises(
+        RuntimeError,
+        match=(
+            'Expected ASGI message "websocket.http.response.body", but got '
+            "'websocket.http.response.start'"
+        ),
+    ):
+        with client.websocket_connect("/"):
+            pass  # pragma: no cover
+
+
+def test_subprotocol(test_client_factory: TestClientFactory) -> None:
     async def app(scope: Scope, receive: Receive, send: Send) -> None:
         websocket = WebSocket(scope, receive=receive, send=send)
         assert websocket["subprotocols"] == ["soap", "wamp"]
@@ -314,7 +423,7 @@ def test_subprotocol(test_client_factory: Callable[..., TestClient]):
         assert websocket.accepted_subprotocol == "wamp"
 
 
-def test_additional_headers(test_client_factory: Callable[..., TestClient]):
+def test_additional_headers(test_client_factory: TestClientFactory) -> None:
     async def app(scope: Scope, receive: Receive, send: Send) -> None:
         websocket = WebSocket(scope, receive=receive, send=send)
         await websocket.accept(headers=[(b"additional", b"header")])
@@ -325,7 +434,7 @@ def test_additional_headers(test_client_factory: Callable[..., TestClient]):
         assert websocket.extra_headers == [(b"additional", b"header")]
 
 
-def test_no_additional_headers(test_client_factory: Callable[..., TestClient]):
+def test_no_additional_headers(test_client_factory: TestClientFactory) -> None:
     async def app(scope: Scope, receive: Receive, send: Send) -> None:
         websocket = WebSocket(scope, receive=receive, send=send)
         await websocket.accept()
@@ -336,7 +445,7 @@ def test_no_additional_headers(test_client_factory: Callable[..., TestClient]):
         assert websocket.extra_headers == []
 
 
-def test_websocket_exception(test_client_factory: Callable[..., TestClient]):
+def test_websocket_exception(test_client_factory: TestClientFactory) -> None:
     async def app(scope: Scope, receive: Receive, send: Send) -> None:
         assert False
 
@@ -346,7 +455,7 @@ def test_websocket_exception(test_client_factory: Callable[..., TestClient]):
             pass  # pragma: no cover
 
 
-def test_duplicate_close(test_client_factory: Callable[..., TestClient]):
+def test_duplicate_close(test_client_factory: TestClientFactory) -> None:
     async def app(scope: Scope, receive: Receive, send: Send) -> None:
         websocket = WebSocket(scope, receive=receive, send=send)
         await websocket.accept()
@@ -359,7 +468,7 @@ def test_duplicate_close(test_client_factory: Callable[..., TestClient]):
             pass  # pragma: no cover
 
 
-def test_duplicate_disconnect(test_client_factory: Callable[..., TestClient]):
+def test_duplicate_disconnect(test_client_factory: TestClientFactory) -> None:
     async def app(scope: Scope, receive: Receive, send: Send) -> None:
         websocket = WebSocket(scope, receive=receive, send=send)
         await websocket.accept()
@@ -373,7 +482,7 @@ def test_duplicate_disconnect(test_client_factory: Callable[..., TestClient]):
             websocket.close()
 
 
-def test_websocket_scope_interface():
+def test_websocket_scope_interface() -> None:
     """
     A WebSocket can be instantiated with a scope, and presents a `Mapping`
     interface.
@@ -382,8 +491,7 @@ def test_websocket_scope_interface():
     async def mock_receive() -> Message:  # type: ignore
         ...  # pragma: no cover
 
-    async def mock_send(message: Message) -> None:
-        ...  # pragma: no cover
+    async def mock_send(message: Message) -> None: ...  # pragma: no cover
 
     websocket = WebSocket(
         {"type": "websocket", "path": "/abc/", "headers": []},
@@ -405,7 +513,7 @@ def test_websocket_scope_interface():
     assert {websocket} == {websocket}
 
 
-def test_websocket_close_reason(test_client_factory: Callable[..., TestClient]) -> None:
+def test_websocket_close_reason(test_client_factory: TestClientFactory) -> None:
     async def app(scope: Scope, receive: Receive, send: Send) -> None:
         websocket = WebSocket(scope, receive=receive, send=send)
         await websocket.accept()
@@ -419,7 +527,7 @@ def test_websocket_close_reason(test_client_factory: Callable[..., TestClient])
         assert exc.value.reason == "Going Away"
 
 
-def test_send_json_invalid_mode(test_client_factory: Callable[..., TestClient]):
+def test_send_json_invalid_mode(test_client_factory: TestClientFactory) -> None:
     async def app(scope: Scope, receive: Receive, send: Send) -> None:
         websocket = WebSocket(scope, receive=receive, send=send)
         await websocket.accept()
@@ -431,7 +539,7 @@ def test_send_json_invalid_mode(test_client_factory: Callable[..., TestClient]):
             pass  # pragma: no cover
 
 
-def test_receive_json_invalid_mode(test_client_factory: Callable[..., TestClient]):
+def test_receive_json_invalid_mode(test_client_factory: TestClientFactory) -> None:
     async def app(scope: Scope, receive: Receive, send: Send) -> None:
         websocket = WebSocket(scope, receive=receive, send=send)
         await websocket.accept()
@@ -443,7 +551,7 @@ def test_receive_json_invalid_mode(test_client_factory: Callable[..., TestClient
             pass  # pragma: nocover
 
 
-def test_receive_text_before_accept(test_client_factory: Callable[..., TestClient]):
+def test_receive_text_before_accept(test_client_factory: TestClientFactory) -> None:
     async def app(scope: Scope, receive: Receive, send: Send) -> None:
         websocket = WebSocket(scope, receive=receive, send=send)
         await websocket.receive_text()
@@ -454,7 +562,7 @@ def test_receive_text_before_accept(test_client_factory: Callable[..., TestClien
             pass  # pragma: nocover
 
 
-def test_receive_bytes_before_accept(test_client_factory: Callable[..., TestClient]):
+def test_receive_bytes_before_accept(test_client_factory: TestClientFactory) -> None:
     async def app(scope: Scope, receive: Receive, send: Send) -> None:
         websocket = WebSocket(scope, receive=receive, send=send)
         await websocket.receive_bytes()
@@ -465,7 +573,7 @@ def test_receive_bytes_before_accept(test_client_factory: Callable[..., TestClie
             pass  # pragma: nocover
 
 
-def test_receive_json_before_accept(test_client_factory: Callable[..., TestClient]):
+def test_receive_json_before_accept(test_client_factory: TestClientFactory) -> None:
     async def app(scope: Scope, receive: Receive, send: Send) -> None:
         websocket = WebSocket(scope, receive=receive, send=send)
         await websocket.receive_json()
@@ -476,7 +584,7 @@ def test_receive_json_before_accept(test_client_factory: Callable[..., TestClien
             pass  # pragma: no cover
 
 
-def test_send_before_accept(test_client_factory: Callable[..., TestClient]):
+def test_send_before_accept(test_client_factory: TestClientFactory) -> None:
     async def app(scope: Scope, receive: Receive, send: Send) -> None:
         websocket = WebSocket(scope, receive=receive, send=send)
         await websocket.send({"type": "websocket.send"})
@@ -487,7 +595,7 @@ def test_send_before_accept(test_client_factory: Callable[..., TestClient]):
             pass  # pragma: nocover
 
 
-def test_send_wrong_message_type(test_client_factory: Callable[..., TestClient]):
+def test_send_wrong_message_type(test_client_factory: TestClientFactory) -> None:
     async def app(scope: Scope, receive: Receive, send: Send) -> None:
         websocket = WebSocket(scope, receive=receive, send=send)
         await websocket.send({"type": "websocket.accept"})
@@ -499,7 +607,7 @@ def test_send_wrong_message_type(test_client_factory: Callable[..., TestClient])
             pass  # pragma: no cover
 
 
-def test_receive_before_accept(test_client_factory: Callable[..., TestClient]):
+def test_receive_before_accept(test_client_factory: TestClientFactory) -> None:
     async def app(scope: Scope, receive: Receive, send: Send) -> None:
         websocket = WebSocket(scope, receive=receive, send=send)
         await websocket.accept()
@@ -512,8 +620,8 @@ def test_receive_before_accept(test_client_factory: Callable[..., TestClient]):
             websocket.send({"type": "websocket.send"})
 
 
-def test_receive_wrong_message_type(test_client_factory: Callable[..., TestClient]):
-    async def app(scope: Scope, receive: Receive, send: Send):
+def test_receive_wrong_message_type(test_client_factory: TestClientFactory) -> None:
+    async def app(scope: Scope, receive: Receive, send: Send) -> None:
         websocket = WebSocket(scope, receive=receive, send=send)
         await websocket.accept()
         await websocket.receive()
